# RTLS Reports

<!-- START doctoc generated TOC please keep comment here to allow auto update -->
<!-- DON'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->

- [Features](#features)
- [Tech Stack](#tech-stack)
- [Architecture Overview](#architecture-overview)
- [Requirements](#requirements)
- [Clone the project](#clone-the-project)
- [Configuration](#configuration)
- [Usage](#usage)
    - [Running the Service Locally](#running-the-service-locally)
    - [Running Tests](#running-tests)
    - [Package Release](#package-release)
- [Conventions](#conventions)
    - [Code style](#code-style)
        - [Formatting rules](#formatting-rules)
        - [Naming rules](#naming-rules)

<!-- END doctoc generated TOC please keep comment here to allow auto update -->

## Features

This GitHub repository contains a backend service that is called as `Reports`.

## Tech Stack

- **Language**: JavaScript
- **Platform**: Node.js
- **Framework**: Express
- **Database**: MariaDB 10.6.18

## Architecture Overview

The component is implementing Node.js service that is by CRON generates reports based on the data generated with RTLS
system. Service implemented as http server.

## Requirements

- Ubuntu 22.04 (Jammy Jellyfish)
- Install [git](https://github.com/rpplabs/rtls-frontend/blob/dev/README.md#install-git)
- Install [Node.js](https://github.com/rpplabs/rtls-frontend/blob/dev/README.md#install-nodejs)

## Clone the project

- Add public ssh key to [your account](https://github.com/settings/keys)

- Clone the repository

```shell
# via https
git clone https://github.com/rpplabs/rtls-reports.git

# OR via ssh key
git clone git@github.com:rpplabs/rtls-reports.git

cd rtls-reports

```

## Configuration

- install server node packages

```shell
npm install

```

## Usage

### Running the Service Locally

- run server

```shell
npm start

```

- The service will be available at: `http://localhost:3200`

### Running Tests

- run unit tests `npm test`

### Package Release

- Reset all current changes, checkout and update current development branch(`dev`)

```shell
git checkout -- ./
git checkout dev
git pull origin dev

```

- Update [CHANGELOG.md](CHANGELOG.md) to add release date to latest version

- Run scripts for version increment

```shell
yarn version:patch

```

- Create new build branch (`master`)

```shell
git branch -D master
git branch master

```

- Make **force push**(for the case if `master` branch already exists from branch differ than `dev`)

```shell
git push origin master -f 

```

- Check build progress
  in [jenkins](https://jenkins.rpplabs.com/job/rtls-web-team/job/rtls-reports/job/master/)
- After build complete package will locate in [repository](https://repo.rpplabs.com/ubuntu/) as `rtls-reports_*`

## Conventions

### Code style

#### Formatting rules

- please, follow the eslint rules described in project (with this purpose setting up your IDE with
  the [.eslintrc](.eslintrc));

#### Naming rules

- please name the variables and constants in a human-readable manner (ex. use `getUsersSql` instead of `sql1`);
- please use the `camelCase` style in name the variables and constants (ex. `getUsersSql` instead of `get_users_sql`);
- please use upper case naming for constants that are not changeable in project (ex. `MAX_ROWS_NUMBER`);
- please use `JavaScript` conventions in naming the classes, constructors(functions) and others entities (classes should
  be named wih the first capital latter);
