"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AlarmsTrackable = void 0;
var _assert = require("assert");
var _utils = require("../utils");
class AlarmsTrackable {
  constructor(trackable, from, to) {
    (0, _assert.strict)(trackable, '"trackable" should be provided');
    (0, _assert.strict)((0, _utils.isValidDate)(from), '"from" should be provided');
    (0, _assert.strict)((0, _utils.isValidDate)(to), '"to" should be provided');
    (0, _assert.strict)(new Date(to) > new Date(from), '"to" should be greater than "from" argument');
    this.from = new Date(from).getTime();
    this.to = new Date(to).getTime();
    this.name = trackable.name;
    this.category = trackable.category;
    this.uid = trackable.uid;
    this.count = 0;
    this.zone_violation_duration = 0;
    this.zone_violations = [];
    this.buffer_violation_duration = 0;
    this.buffer_violations = [];
    this.emergency_button = {
      count: 0,
      accumulated_duration: 0
    };
    this.uncertified_operation = {
      count: 0,
      accumulated_duration: 0
    };
  }
  _alarmDuration(alarm) {
    if (!alarm.time_set) return 0;
    const time_set = new Date(alarm.time_set).getTime();
    const time_clear = alarm.time_clear ? new Date(alarm.time_clear).getTime() : new Date().getTime();
    const start = this.from > time_set ? this.from : time_set;
    const end = this.to > time_clear ? time_clear : this.to;
    return end - start;
  }
  set zoneViolationAlarm(alarm) {
    this.count += 1;
    const zone_violation = this.zone_violations.find(item => item.zone_uid === alarm.additional_info.zone_uid);
    const duration = this._alarmDuration(alarm);
    this.zone_violation_duration += duration;
    if (!zone_violation) {
      this.zone_violations.push({
        zone_uid: alarm.additional_info.zone_uid,
        zone_name: alarm.additional_info.zone_name,
        severity: alarm.additional_info.zone_alarm_level,
        zone_type: alarm.additional_info.zone_violation_type,
        count: 1,
        accumulated_duration: duration
      });
    } else {
      zone_violation.count += 1;
      zone_violation.accumulated_duration += duration;
    }
  }
  set bufferViolationAlarm(alarm) {
    this.count += 1;
    const buffer_violation = this.buffer_violations.find(item => item.zone_uid === alarm.additional_info.zone_uid);
    const duration = this._alarmDuration(alarm);
    this.buffer_violation_duration += duration;
    if (!buffer_violation) {
      this.buffer_violations.push({
        zone_uid: alarm.additional_info.zone_uid,
        zone_name: alarm.additional_info.zone_name,
        severity: alarm.additional_info.zone_alarm_level,
        zone_type: alarm.additional_info.zone_violation_type,
        count: 1,
        accumulated_duration: duration
      });
    } else {
      buffer_violation.count += 1;
      buffer_violation.accumulated_duration += duration;
    }
  }
  set emergencyButtonAlarm(alarm) {
    this.count += 1;
    this.emergency_button.count += 1;
    this.emergency_button.accumulated_duration += this._alarmDuration(alarm);
  }
  set uncertifiedOpAlarm(alarm) {
    this.count += 1;
    this.uncertified_operation.count += 1;
    this.uncertified_operation.accumulated_duration += this._alarmDuration(alarm);
  }
}
exports.AlarmsTrackable = AlarmsTrackable;