"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TravelDistanceTrackable = void 0;
var _utils = require("../utils");
class TravelDistanceTrackable {
  constructor(trackable, report) {
    this.prevPosition = null;
    this.currPosition = null;
    this.uid = trackable.uid;
    this._name = trackable.name;
    this.category = trackable.category;
    this.startDate = report.startDate;
    this.endDate = report.endDate;
    this.step = report.step;
    this.reportFirstRangesShift = report.reportFirstRangesShift;
    this.ranges = report.ranges;
    this.distances = report.ranges.map(() => 0);
    this.totalDistance = 0;
  }
  set name(name) {
    this._name = name;
  }
  get name() {
    return this._name;
  }
  get data() {
    return {
      uid: this.uid,
      name: this._name,
      category: this.category,
      distances: this.distances,
      totalDistance: this.totalDistance
    };
  }
  getPositionRangeIndex(posTs) {
    return (0, _utils.getPositionRangeIndex)(posTs, this.startDate, this.reportFirstRangesShift, this.step);
  }
  setPrevPosition(position) {
    this.prevPosition = position;
  }
  setCurrentPosition(position) {
    this.currPosition = position;
  }
  getRangeTimestamp(rangeIndex) {
    return this.ranges[rangeIndex];
  }
  addDistanceToRange(rangeIndex, deltaDistance) {
    if (this.distances[rangeIndex] === undefined) {
      this.distances[rangeIndex] = 0;
    }
    this.distances[rangeIndex] += deltaDistance;
  }
  incrementTotalDistance(deltaDistance) {
    this.totalDistance += deltaDistance;
  }
  handlePositionsInOneRange(currentPosRangeIndex, deltaDistance) {
    this.addDistanceToRange(currentPosRangeIndex, deltaDistance);
    this.incrementTotalDistance(deltaDistance);
  }
  handlePositionsInDiffRanges(startRangeIndex, rangesNumber, distance) {
    const deltaTime = this.currPosition.ts - this.prevPosition.ts;
    for (let i = 0; i <= rangesNumber; i += 1) {
      const firstRange = i === 0;
      const lastRange = i === rangesNumber;
      let rangeDistanceWeight = 0;
      if (firstRange) {
        rangeDistanceWeight = (this.getRangeTimestamp(startRangeIndex) - this.prevPosition.ts) / deltaTime;
      } else if (lastRange) {
        const prevRangeIndex = startRangeIndex + (rangesNumber - 1);
        rangeDistanceWeight = (this.currPosition.ts - this.getRangeTimestamp(prevRangeIndex)) / deltaTime;
      } else {
        rangeDistanceWeight = this.step / deltaTime;
      }
      this.addDistanceToRange(startRangeIndex + i, rangeDistanceWeight * distance);
      this.incrementTotalDistance(rangeDistanceWeight * distance);
    }
  }
  handlePosition(position) {
    if (!this.prevPosition) {
      this.setPrevPosition(position);
      return;
    }
    this.setCurrentPosition(position);
    const prevPosRangeIndex = this.getPositionRangeIndex(this.prevPosition.ts);
    const currentPosRangeIndex = this.getPositionRangeIndex(this.currPosition.ts);
    const rangesNumber = currentPosRangeIndex - prevPosRangeIndex;
    const positionsInOneRange = rangesNumber === 0;
    const deltaDistance = (0, _utils.distanceBetween)(this.currPosition, this.prevPosition);
    if (positionsInOneRange) {
      this.handlePositionsInOneRange(currentPosRangeIndex, deltaDistance);
    } else {
      this.handlePositionsInDiffRanges(prevPosRangeIndex, rangesNumber, deltaDistance);
    }
    this.setPrevPosition(position);
  }
}
exports.TravelDistanceTrackable = TravelDistanceTrackable;