"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
var _utils = require("../utils");
var _repository = require("../repository");
var _logger = _interopRequireDefault(require("../logger"));
async function getActivityByTimeFrame(start, end, location) {
  const sublocationUids = (await _repository.LocationRepo.getSublocations(location.uid)).map(sl => sl.uid);
  const idleEvents = await _repository.EventRepo.getIdleEvents(start, end, sublocationUids);
  const heartbeats = await _repository.HeartbeatRepo.getList(start, end, sublocationUids);
  const trackables = await _repository.HeartbeatRepo.getObjects(start, end, sublocationUids);
  const objUids = trackables.map(row => row.uid);
  const associations = await _repository.AssociationRepo.getList(objUids, start, end);
  const activities = (0, _utils.getTrackablesActivity)(trackables, associations, heartbeats, idleEvents, start, end);
  return {
    uid: location.uid,
    trackableObjects: activities
  };
}

/**
 * Description
 * @param {String} start
 * @param {String} end
 * @param {Object} location
 * @property {String} location.shift
 * @property {String} location.name
 * @property {String} location.uid
 * @param {Object} options
 * @return {Promise<{
 *      site_name: string,
 *      uid: string,
 *      data: string,
 *      shift: string,
 *      start: string
 *      project_uid: string,
 *      end: string,
 *      site_uid: string,
 *      project_name: string,
 *      type: string
 * }>}
 */
async function combineReport(start, end, location, options) {
  const activity = await getActivityByTimeFrame(start, end, location);
  return {
    shift: JSON.stringify(location.shift),
    project_name: options.project_name,
    project_uid: options.project_uid,
    data: JSON.stringify(activity),
    type: 'attendance_report',
    site_name: location.name,
    site_uid: location.uid,
    uid: (0, _utils.getUUID)(),
    start,
    end
  };
}
async function main(options) {
  // console.log('[rtls-report][Report Attendance] Report ', JSON.stringify(options));
  for (let i = 0; i < options.locations.length; i += 1) {
    const location = options.locations[i];
    const {
      start,
      end
    } = (0, _utils.getShiftTimeframe)(location.shift, location.timezone);
    const result = await combineReport(start, end, location, options);
    try {
      await _repository.ReportRepo.saveDw(result);
      // console.log('[rtls-report][Report Attendance] Report ', result.uid, 'saved in rpp_dw_prod');
    } catch (err) {
      _logger.default.error(err);
      await _repository.ReportRepo.save(result);
      // console.log('[rtls-report][Report Attendance] Report ', result.uid, 'saved in dr2_prod');
    }
  }
}

/**
 * Description
 * @param {Object} report
 * @property {String} report.types ['attendance_report', 'alarms', 'distance', 'zone_activity']
 * @property {String} report.project_uid
 * @property {String} report.project_name
 * @property {String} report.timezone
 * @property {Array<Object>} report.locations
 * @return {Promise<void>}
 */
async function handleMessage(report) {
  try {
    await main(report);
    _logger.default.info('[Report Attendance] Done');
    process.exit(0);
  } catch (err) {
    _logger.default.error('[Report Attendance]: Error');
    console.error(err);
    _logger.default.error(err);
    process.exit(1);
  }
}
process.on('message', handleMessage);
process.on('uncaughtException', reason => {
  _logger.default.error('[Report Attendance]: uncaughtException');
  _logger.default.error('[Report Attendance]: reason ', JSON.stringify(reason));
  process.exit(1);
});
process.on('unhandledRejection', reason => {
  _logger.default.error('[Report Attendance]: unhandledRejection');
  _logger.default.error('[Report Attendance]: reason ', JSON.stringify(reason));
  process.exit(1);
});