"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
var _path = _interopRequireDefault(require("path"));
var _child_process = _interopRequireDefault(require("child_process"));
const workers = [];
function handleProcessExit() {
  workers.forEach(worker => worker.kill());
}
process.on('exit', handleProcessExit);
class ReportWorker {
  constructor(report) {
    this.report = report;
    workers.push(this);
  }
  handleError(reject, err) {
    console.error('Report Generation Error', err);
    reject(err);
  }
  handleExit(resolve, code) {
    if (code) {
      const e = new Error(`Worker exit with code = ${code}`);
      console.error(e);
    }
    resolve();
  }
  get files() {
    return this.report.types.map(type => _path.default.join(__dirname, '../reports', `${type.toLowerCase()}_worker.js`));
  }
  async runReport(reportFile) {
    return new Promise((resolve, reject) => {
      this.worker = _child_process.default.fork(reportFile, {
        execArgv: ['--max-old-space-size=2048']
      });
      this.worker.on('error', this.handleError.bind(this, reject));
      this.worker.on('exit', this.handleExit.bind(this, resolve));
      this.worker.send(this.report);
    });
  }
  async run() {
    for (const file of this.files) {
      await this.runReport(file);
    }
  }
  kill() {
    if (this.worker) {
      this.worker.kill();
    }
  }
}
module.exports = ReportWorker;