"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.distanceBetween = distanceBetween;
exports.getPositionRangeIndex = getPositionRangeIndex;
exports.getRanges = getRanges;
exports.getShiftTimeframe = getShiftTimeframe;
exports.isOverlapped = isOverlapped;
var _moment = _interopRequireDefault(require("moment"));
var _cronParser = _interopRequireDefault(require("cron-parser"));
var _timeUtils = require("./time-utils");
/**
 * Get start/end time for shift by parsing cron pattern
 * @param {Object} shift
 * @property {String} shift.start - cron pattern '0 0 * * *'
 * @property {String} shift.end - cron pattern '59 23 * * *'
 * @param {String} timezone - project timezone
 * @return {{start: string, end: string}}
 */
function getShiftTimeframe(shift, timezone) {
  const _start = (0, _moment.default)(_cronParser.default.parseExpression(shift.start, {
    tz: timezone
  }).prev().toISOString());
  const _end = (0, _moment.default)(_cronParser.default.parseExpression(shift.end, {
    tz: timezone
  }).prev().toISOString());
  if (_end.isBefore(_start)) {
    _end.add(24, 'hours');
  }
  if (_start.isSame(_end)) {
    _start.add(-24, 'hours');
  }
  const [min_str, hour_str] = shift.end.split(' ');
  const min = Number(min_str);
  const hour = Number(hour_str);
  if (hour === 23 && min >= 55 && min <= 59) {
    _end.add(-24, 'hours');
    _start.add(-24, 'hours');
  }
  const start = (0, _timeUtils.timeFormatMySql)(_start.valueOf());
  const end = (0, _timeUtils.timeFormatMySql)(_end.valueOf());
  return {
    start,
    end
  };
}
function getRanges(startDate, endDate, step, reportFirstRangesShift = 0) {
  function _getRanges(start, end) {
    const ranges = [];
    const rangeNumber = Math.ceil((end - start) / step);
    for (let i = 1; i <= rangeNumber; i += 1) {
      if (start + i * step > end) {
        ranges.push(end);
      } else {
        ranges.push(start + i * step);
      }
    }
    return ranges;
  }
  if (reportFirstRangesShift) {
    const startWithShift = startDate + reportFirstRangesShift;
    if (startWithShift > endDate) {
      return [endDate];
    }
    const ranges = _getRanges(startWithShift, endDate);
    return [startWithShift, ...ranges];
  }
  return _getRanges(startDate, endDate);
}
function getPositionRangeIndex(posTs, startDate, shift = 0, step) {
  if (posTs <= startDate + shift) {
    return 0;
  }
  const indexAdjustment = shift ? 1 : 0;
  return Math.floor((posTs - (startDate + shift)) / step) + indexAdjustment;
}
function distanceBetween(pos_one, pos_two) {
  if (isNaN(pos_one.x) || isNaN(pos_one.y) || isNaN(pos_two.x) || isNaN(pos_two.y)) {
    return 0;
  }
  const deltaX = pos_one.x - pos_two.x;
  const deltaY = pos_one.y - pos_two.y;
  return Math.ceil(Math.sqrt(deltaX * deltaX + deltaY * deltaY));
}
function isOverlapped(rangeOne, rangeTwo) {
  return (0, _moment.default)(rangeOne.start_time).isBefore(rangeTwo.stop_time) && (0, _moment.default)(rangeOne.stop_time).isAfter(rangeTwo.start_time);
}