"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getTrackablesActivity = getTrackablesActivity;
var _moment = _interopRequireDefault(require("moment"));
var _reports = require("./reports");
var _common = require("../common");
function getIdleActiveIntervals(_idle_events = [], start, end) {
  const idle_events = _idle_events.filter(event => (0, _reports.isOverlapped)(event, {
    start_time: start,
    stop_time: end
  }));
  const active_period = {
    start,
    end
  };
  const eventsCount = idle_events.length;
  if (idle_events.length === 0) {
    return {
      idle_periods: [],
      totalIdle: 0,
      active_periods: [active_period],
      totalActive: end.diff(start)
    };
  }
  return idle_events.reduce((memo, idle_event, index) => {
    const idle_start = (0, _moment.default)(idle_event.start_time);
    const idle_stop = (0, _moment.default)(idle_event.stop_time);
    if (idle_start.isBefore(end) && idle_stop.isAfter(start)) {
      const idle_start_time = idle_start.isAfter(start) ? idle_start : start;
      const idle_stop_time = idle_stop.isAfter(end) ? end : idle_stop;
      if (active_period.start.isBefore(idle_start_time)) {
        active_period.end = idle_start_time;
        memo.active_periods.push({
          ...active_period
        });
        memo.totalActive += active_period.end.diff(active_period.start);
        active_period.start = idle_stop_time;
        active_period.end = end;
      } else if (idle_stop_time.isBefore(end)) {
        active_period.start = idle_stop_time;
      }
      if (eventsCount === index + 1 && active_period.start.isSame(idle_stop_time) && active_period.end.isAfter(active_period.start)) {
        memo.active_periods.push({
          ...active_period
        });
        memo.totalActive += active_period.end.diff(active_period.start);
      }
      memo.idle_periods.push({
        start: idle_start_time,
        end: idle_stop_time
      });
      memo.totalIdle += idle_stop_time.diff(idle_start_time);
    }
    return memo;
  }, {
    idle_periods: [],
    totalIdle: 0,
    active_periods: [],
    totalActive: 0
  });
}
function getUnresponsiveIntervals(checkInOutList = [], responsiveIntervals = []) {
  const result = [];
  let totalUnresponsive = 0;
  checkInOutList.forEach((_checkInOut, i) => {
    const checkInOut = {
      ..._checkInOut
    };
    const assoc_intervals = [];
    let isContainResponsivePeriod = false;
    let hasInvalidIntervals = false;
    responsiveIntervals.forEach(interval => {
      if (!interval.start.isBetween(checkInOut.start, checkInOut.end, null, '[]') && !interval.end.isBetween(checkInOut.start, checkInOut.end, null, '[]')) {
        if (interval.start.isBefore(checkInOut.start) && interval.end.isAfter(checkInOut.end)) hasInvalidIntervals = true;
        return;
      }
      const lastUnresponsive = assoc_intervals.pop();
      if (lastUnresponsive && lastUnresponsive.i === i) {
        checkInOut.start = lastUnresponsive.start;
      }
      if (interval.start.isBetween(checkInOut.start, checkInOut.end, null, '[]') || interval.end.isBetween(checkInOut.start, checkInOut.end, null, '[]')) {
        isContainResponsivePeriod = true;
        if (checkInOut.start.isBefore(interval.start)) assoc_intervals.push({
          start: checkInOut.start,
          end: interval.start,
          i
        });
        if (interval.end.isBefore(checkInOut.end)) assoc_intervals.push({
          start: interval.end,
          end: checkInOut.end,
          i
        });
      }
    });
    if (!isContainResponsivePeriod && !hasInvalidIntervals) {
      assoc_intervals.push({
        start: checkInOut.start,
        end: checkInOut.end,
        i
      });
    }
    totalUnresponsive += assoc_intervals.reduce((sum, interval) => interval.end.diff(interval.start), 0);
    result.push(...assoc_intervals);
  });
  return {
    unresponsive_periods: result,
    totalUnresponsive
  };
}
function splitHbByTrackable(heartbeats = [], _start, _end) {
  const start = (0, _moment.default)(_start);
  const end = (0, _moment.default)(_end);
  const now = (0, _moment.default)();
  const trackables = {};
  for (let i = 0; i < heartbeats.length; i += 1) {
    const heartbeat = heartbeats[i];
    const hbEnter = (0, _moment.default)(heartbeat.time_enter);
    const hbLeave = heartbeat.time_leave ? (0, _moment.default)(heartbeat.time_leave) : now;
    const hb_start = hbEnter.isBefore(start) ? start : hbEnter;
    const hb_end = hbLeave.isAfter(end) ? end : hbLeave;
    if (!trackables[heartbeat.obj_uid]) {
      trackables[heartbeat.obj_uid] = [];
    }
    trackables[heartbeat.obj_uid].push({
      start: hb_start,
      end: hb_end
    });
  }
  return trackables;
}
function splitIdleEventsByTrackable(idle_events) {
  const trackables = {};
  for (let i = 0; i < idle_events.length; i += 1) {
    const idle_event = idle_events[i];
    if (!trackables[idle_event.trackable_uid]) {
      trackables[idle_event.trackable_uid] = [];
    }
    trackables[idle_event.trackable_uid].push(idle_event);
  }
  return trackables;
}
function isAssignedStatus(associationPeriods, hbGapStart, hbGapEnd) {
  let result = false;
  for (let i = 0; i < associationPeriods.length; i += 1) {
    if (hbGapStart.isBetween(associationPeriods[i].start, associationPeriods[i].end)) {
      if (hbGapEnd.isBetween(associationPeriods[i].start, associationPeriods[i].end)) {
        result = true;
      }
      break;
    }
  }
  return result;
}

// heartbeats should be sorted. in current implementation sorting in db by order by
function getResponsiveIntervals(heartbeats, start, end, checkInOutList) {
  const trackables = splitHbByTrackable(heartbeats, start, end);
  const result = {};
  for (const key in trackables) {
    const hbCount = trackables[key].length;
    const action = {};
    result[key] = trackables[key].reduce((memo, hb, index) => {
      if (!action.start) {
        action.start = hb.start;
        action.end = hb.end;
      }
      if (hb.start.isAfter(action.end) && (hb.start.diff(action.end) > _common.HEARTBEAT.UNRESPONSIVE_DURATION || !isAssignedStatus(checkInOutList[key], action.end, hb.start))) {
        memo.responsiveIntervals.push({
          ...action
        });
        memo.totalOnSite += action.end.diff(action.start);
        action.start = hb.start;
        action.end = hb.end;
      } else {
        action.end = hb.end;
      }
      if (hbCount === index + 1) {
        memo.responsiveIntervals.push({
          ...action,
          end: hb.end
        });
        memo.totalOnSite += hb.end.diff(action.start);
      }
      return memo;
    }, {
      responsiveIntervals: [],
      totalOnSite: 0
    });
  }
  return result;
}
function getTrackablesCheckInOut(associations, _start, _end) {
  const now = (0, _moment.default)();
  const start = (0, _moment.default)(_start);
  const end = (0, _moment.default)(_end).isAfter(now) ? now : (0, _moment.default)(_end);
  const result = {};
  for (let i = 0; i < associations.length; i += 1) {
    const association = associations[i];
    const check_in = (0, _moment.default)(association.created_at);
    const check_out = (0, _moment.default)(association.removed_at);
    if (check_in.isBefore(end) && check_out.isAfter(start)) {
      const check_in_out = {
        start: check_in,
        end: check_out
      };
      if (!result[association.trob_uid]) {
        result[association.trob_uid] = [];
      }
      if (check_in.isBefore(start)) {
        check_in_out.start = start;
      }
      if (check_out.isAfter(end)) {
        check_in_out.end = end;
      }
      result[association.trob_uid].push(check_in_out);
    }
  }
  return result;
}
function getTrackablesActivity(trackables, associations, heartbeats, idle_events, start, end) {
  const checkInOutList = getTrackablesCheckInOut(associations, start, end);
  const allResponsiveIntervals = getResponsiveIntervals(heartbeats, start, end, checkInOutList);
  const trackablesIdleEvents = splitIdleEventsByTrackable(idle_events);
  const activities = {};
  for (const key in allResponsiveIntervals) {
    const trackable = trackables.find(tko => tko.uid === key);
    activities[key] = {};
    const {
      responsiveIntervals
    } = allResponsiveIntervals[key];
    const {
      unresponsive_periods,
      totalUnresponsive
    } = getUnresponsiveIntervals(checkInOutList[key], responsiveIntervals);
    activities[key] = responsiveIntervals.reduce((memo, interval) => {
      const idleActiveIntervals = getIdleActiveIntervals(trackablesIdleEvents[key], interval.start, interval.end);
      memo.idle_periods.push(...idleActiveIntervals.idle_periods);
      memo.active_periods.push(...idleActiveIntervals.active_periods);
      memo.totalIdle += idleActiveIntervals.totalIdle;
      memo.totalActive += idleActiveIntervals.totalActive;
      return memo;
    }, {
      idle_periods: [],
      totalIdle: 0,
      active_periods: [],
      totalActive: 0
    });
    activities[key].unresponsive_periods = unresponsive_periods;
    activities[key].totalUnresponsive = totalUnresponsive;
    activities[key].name = trackable ? trackable.name : '';
    activities[key].category = trackable ? trackable.category : '';
  }
  return activities;
}