"use strict";

const reports = require('./reports');
const NewYorkTimeZonePlus5 = 'America/New_York';
const KievTimeZoneMinus2 = 'Europe/Kiev';
describe('Utils.', () => {
  describe('reports', () => {
    describe('getShiftTimeframe America/New_York', () => {
      beforeAll(() => {
        Date.now = jest.fn(() => 1641359040000); // 2022-01-05T05:04:00.000Z
      });

      const testCases = [{
        name: 'America/New_York all day',
        timezone: NewYorkTimeZonePlus5,
        shift: {
          start: '0 0 * * *',
          end: '59 23 * * *'
        },
        expectedResult: {
          start: '2022-01-04 05:00:00.000',
          end: '2022-01-05 04:59:00.000'
        }
      }, {
        name: 'America/New_York 15 min',
        timezone: NewYorkTimeZonePlus5,
        shift: {
          start: '0 0 * * *',
          end: '15 0 * * *'
        },
        expectedResult: {
          start: '2022-01-05 05:00:00.000',
          end: '2022-01-05 05:15:00.000'
        }
      }, {
        name: 'America/New_York 4 hour',
        timezone: NewYorkTimeZonePlus5,
        shift: {
          start: '0 0 * * *',
          end: '0 4 * * *'
        },
        expectedResult: {
          start: '2022-01-05 05:00:00.000',
          end: '2022-01-05 09:00:00.000'
        }
      }, {
        name: 'America/New_York 6 hour',
        timezone: NewYorkTimeZonePlus5,
        shift: {
          start: '0 0 * * *',
          end: '0 6 * * *'
        },
        expectedResult: {
          start: '2022-01-05 05:00:00.000',
          end: '2022-01-05 11:00:00.000'
        }
      }, {
        name: 'America/New_York 18 hour',
        timezone: NewYorkTimeZonePlus5,
        shift: {
          start: '0 0 * * *',
          end: '0 18 * * *'
        },
        expectedResult: {
          start: '2022-01-05 05:00:00.000',
          end: '2022-01-05 23:00:00.000'
        }
      }, {
        name: 'America/New_York 19 hour',
        timezone: NewYorkTimeZonePlus5,
        shift: {
          start: '0 0 * * *',
          end: '0 19 * * *'
        },
        expectedResult: {
          start: '2022-01-05 05:00:00.000',
          end: '2022-01-06 00:00:00.000'
        }
      }, {
        name: 'America/New_York 20 hour',
        timezone: NewYorkTimeZonePlus5,
        shift: {
          start: '0 0 * * *',
          end: '0 20 * * *'
        },
        expectedResult: {
          start: '2022-01-05 05:00:00.000',
          end: '2022-01-06 01:00:00.000'
        }
      }];
      testCases.forEach(({
        name,
        shift,
        timezone,
        expectedResult
      }) => {
        it(`${name}`, () => {
          const result = reports.getShiftTimeframe(shift, timezone);
          expect(result).toEqual(expectedResult);
        });
      });
    });
    describe('getShiftTimeframe Europe/Kiev', () => {
      beforeAll(() => {
        Date.now = jest.fn(() => 1641333840000); // 2022-01-04T22:04:00.000Z
      });

      const testCases = [{
        name: 'Europe/Kiev all day',
        timezone: KievTimeZoneMinus2,
        shift: {
          start: '0 0 * * *',
          end: '59 23 * * *'
        },
        expectedResult: {
          start: '2022-01-03 22:00:00.000',
          end: '2022-01-04 21:59:00.000'
        }
      }, {
        name: 'Europe/Kiev 1 hour',
        timezone: KievTimeZoneMinus2,
        shift: {
          start: '0 0 * * *',
          end: '0 1 * * *'
        },
        expectedResult: {
          start: '2022-01-04 22:00:00.000',
          end: '2022-01-04 23:00:00.000'
        }
      }, {
        name: 'Europe/Kiev 2 hour',
        timezone: KievTimeZoneMinus2,
        shift: {
          start: '0 0 * * *',
          end: '0 2 * * *'
        },
        expectedResult: {
          start: '2022-01-04 22:00:00.000',
          end: '2022-01-05 00:00:00.000'
        }
      }, {
        name: 'Europe/Kiev 12 hour',
        timezone: KievTimeZoneMinus2,
        shift: {
          start: '0 0 * * *',
          end: '0 12 * * *'
        },
        expectedResult: {
          start: '2022-01-04 22:00:00.000',
          end: '2022-01-05 10:00:00.000'
        }
      }, {
        name: 'Europe/Kiev 23 hour',
        timezone: KievTimeZoneMinus2,
        shift: {
          start: '0 0 * * *',
          end: '0 23 * * *'
        },
        expectedResult: {
          start: '2022-01-04 22:00:00.000',
          end: '2022-01-05 21:00:00.000'
        }
      }];
      testCases.forEach(({
        name,
        shift,
        timezone,
        expectedResult
      }) => {
        it(`${name}`, () => {
          const result = reports.getShiftTimeframe(shift, timezone);
          expect(result).toEqual(expectedResult);
        });
      });
    });
  });
});