import fs from 'fs';
import Promise from 'bluebird';
import mysql from 'mysql';
import * as path from 'path';
import logger from '../logger';

const config_env = (process.env.RTLS_FRONTEND_ENV || 'PROD').toUpperCase();

function makePool(keyFile, env) {
    if (!fs.existsSync(keyFile)) {
        logger.error(`Db config file "${keyFile}" not found`);
        process.exit(-1);
    }
    try {
        const db_config = JSON.parse(fs.readFileSync(keyFile, 'utf8'));
        const pool = mysql.createPool(db_config[env.toLowerCase()]);
        Promise.promisifyAll(mysql);
        Promise.promisifyAll(require('mysql/lib/Connection').prototype);
        Promise.promisifyAll(require('mysql/lib/Pool').prototype);

        return pool;
    } catch (e) {
        logger.error(`Fails to read db config file "${keyFile}"`);
        logger.error(e);
        process.exit(-1);
    }
    return false;
}

const dbConfigPath = path.join(__dirname, '../../../database.json');
const Pool = makePool(dbConfigPath, config_env);
const dwPool = makePool(dbConfigPath, 'dw');

module.exports = {
    Pool,
    dwPool,
};
