import winston from 'winston';

const options = {
    level: 'info',
    format: winston.format.combine(
        winston.format.colorize({ all: true }),
        winston.format.timestamp({ format: 'YYYY/MM/DD HH:mm:ss' }),
        winston.format.printf((info) => `[${info.timestamp}] [${info.service}] ${info.level}: ${info.message}`)
    ),
    defaultMeta: { service: 'rtls-reports' },
    transports: [new winston.transports.Console()],
};
// todo: logger apply only 1 argument, all other are dropped out
//  apply logger implemented on rtls-server with rotate file and console transport
class Logger {
    #logger = null;

    constructor() {
        this.#logger = winston.createLogger(options);
        this.#generateWrappedMethods();
    }

    #generateWrappedMethods = () => {
        const methods = ['debug', 'info', 'warn', 'error'];

        for (const method of methods) {
            this[method] = (...args) => {
                this.#logger[method](...args);
            };
        }
    };
}

export default Logger;
