import { dwPool, Pool } from '../../db/Pool';
import { getZoneEventsSql, getIdleEventsSql } from './sqls';

export class EventRepo {
    static async _stream(sql) {
        return new Promise((resolve, reject) => {
            dwPool.getConnection(async (err, connection) => {
                if (err) {
                    reject(err);
                }
                const releaseConnection = () => {
                    if (connection) {
                        connection.release();
                        connection = null;
                    }
                };
                const query = connection.query(sql);
                resolve(
                    query
                        .stream()
                        .on('error', (e) => {
                            releaseConnection();
                            console.error(e);
                            reject(e);
                        })
                        .on('end', () => {
                            releaseConnection();
                        })
                );
            });
        });
    }

    static async getZoneEventsStream(from, to, sublocations) {
        const zoneEventsSql = getZoneEventsSql(from, to, sublocations);
        return this._stream(zoneEventsSql);
    }

    static async getIdleEvents(from, to, sublocation_uids) {
        if (!Array.isArray(sublocation_uids) || sublocation_uids.length === 0) {
            return [];
        }
        return Pool.queryAsync(getIdleEventsSql(), [to, from, sublocation_uids]);
    }
}
