import http from 'http';
import { Puller } from './system/Puller';
import restAPI from './rest-api';
import logger from './logger';

const port = Number(process.env.PORT || 3200);
const server = http.createServer(restAPI);
server.listen(port, onListening);

async function onListening() {
    logger.info('Listening on port: ' + port);
    const puller = new Puller();
    await puller.pullProjects();
}

async function exit() {
    if (!server) return;
    server.close();
    process.exit(0);
}

process.on('SIGTERM', async () => {
    logger.error('SIGTERM signal caught');
    await exit();
});

process.on('SIGINT', async () => {
    logger.error('SIGINT signal caught');
    await exit();
});

process.on('unhandledRejection', (error) => {
    logger.error({
        type: 'UnhandledRejection',
        error: error.stack,
    });
});

process.on('uncaughtException', (error) => {
    logger.error({
        type: 'UncaughtException',
        error: error.stack,
    });
});
