import path from 'path';
import child_process from 'child_process';

const workers = [];

function handleProcessExit() {
    workers.forEach((worker) => worker.kill());
}

process.on('exit', handleProcessExit);

class ReportWorker {
    constructor(report) {
        this.report = report;
        workers.push(this);
    }

    handleError(reject, err) {
        console.error('Report Generation Error', err);
        reject(err);
    }

    handleExit(resolve, code) {
        if (code) {
            const e = new Error(`Worker exit with code = ${code}`);
            console.error(e);
        }
        resolve();
    }

    get files() {
        return this.report.types.map((type) => path.join(__dirname, '../reports', `${type.toLowerCase()}_worker.js`));
    }

    async runReport(reportFile) {
        return new Promise((resolve, reject) => {
            this.worker = child_process.fork(reportFile, {
                execArgv: ['--max-old-space-size=2048'],
            });
            this.worker.on('error', this.handleError.bind(this, reject));
            this.worker.on('exit', this.handleExit.bind(this, resolve));
            this.worker.send(this.report);
        });
    }

    async run() {
        for (const file of this.files) {
            await this.runReport(file);
        }
    }

    kill() {
        if (this.worker) {
            this.worker.kill();
        }
    }
}

module.exports = ReportWorker;
